/* 
 *  Stratégies Numériques
 *  http://www.strat-num.fr/dr2d/
 *  
 */

"use strict";

// First, checks if it isn't implemented yet.
if (!String.prototype.format) {
  String.prototype.format = function() {
    var args = arguments;
    return this.replace(/{(\d+)}/g, function(match, number) { 
      return typeof args[number] !== 'undefined'
        ? args[number]
        : match
      ;
    });
  };
}

(function () {
    "use strict";

    Function.prototype.bindMe = function () {
        var fn = this;
        return function () {
            return fn.apply(DR2D_WSAPI, arguments);
        };
    };
    
   function isFunction(functionToCheck) {
     var getType = {};
     return functionToCheck 
             && getType.toString.call(functionToCheck) === '[object Function]';
   };
   
    /**
     * @namespace DR2D_WSAPI
     * 
     * @property {function}     onWsOpen                - function handler when webSocket is opened
     * @property {function}     onWsMessage             - function handler when webSocket received a message
     * @property {function}     onWsClose               - function handler when webSocket has closed
     * @property {function}     onWsError               - function handler when webSocket has raised an error
     * 
     * @property {object}       host                    - the uri value that describe host
     * @property {function}     connect                 - call this to connect to host
     * @property {function}     disconnect              - call this to disconnect from host
     * 
     * @property {function}     sendSms1                - to send an SMS
     * @property {function}     extractReplyFromCall    - helper to build a reply
     * @property {function}     sendReply               - To send a reply
     */
    var DR2D_WSAPI = 
    {
        _writeToLog: function(o1)
        {
          console.log("DR2D_WSAPI_DEBUG> "+o1);  
        },

        _ws: null,          //> the WebSocket instance

        host: null,         //> the host uri

        /* the four callbacks to implement */
        onWsOpen: null,     //> the WebSocket has opened
        onWsMessage: null,  //> the WebSocket received a message
        onWsClose: null,    //> the WebSocket has closed
        onWsError: null,    //> the WebSocket has raised an error
        
        connect: function()
        {
          console.assert(!(this._ws),"heu ....");
          if(this._ws!==null) this.disconnect();
          
          this._writeToLog("try to connect to " + this.host);
          this._ws = new WebSocket(this.host);

          this._ws.onopen = function (evt) {
            this._writeToLog("this._ws.onopen");
            if(isFunction(this.onWsOpen)) this.onWsOpen(evt);
          }.bindMe();

          this._ws.onmessage = function (evt) {
            //this._writeToLog("this._ws.onmessage:"+evt.data);
            if(isFunction(this.onWsMessage)) this.onWsMessage(evt);
          }.bindMe();

          this._ws.onclose = function (event) {
            var reason = event.reason;
            if(reason==''){
                // See http://tools.ietf.org/html/rfc6455#section-7.4.1
                if (event.code == 1000)
                    reason = "Normal closure, meaning that the purpose for which the connection was established has been fulfilled.";
                else if(event.code == 1001)
                    reason = "An endpoint is \"going away\", such as a server going down or a browser having navigated away from a page.";
                else if(event.code == 1002)
                    reason = "An endpoint is terminating the connection due to a protocol error";
                else if(event.code == 1003)
                    reason = "An endpoint is terminating the connection because it has received a type of data it cannot accept (e.g., an endpoint that understands only text data MAY send this if it receives a binary message).";
                else if(event.code == 1004)
                    reason = "Reserved. The specific meaning might be defined in the future.";
                else if(event.code == 1005)
                    reason = "No status code was actually present.";
                else if(event.code == 1006)
                   reason = "The connection was closed abnormally, e.g., without sending or receiving a Close control frame";
                else if(event.code == 1007)
                    reason = "An endpoint is terminating the connection because it has received data within a message that was not consistent with the type of the message (e.g., non-UTF-8 [http://tools.ietf.org/html/rfc3629] data within a text message).";
                else if(event.code == 1008)
                    reason = "An endpoint is terminating the connection because it has received a message that \"violates its policy\". This reason is given either if there is no other sutible reason, or if there is a need to hide specific details about the policy.";
                else if(event.code == 1009)
                   reason = "An endpoint is terminating the connection because it has received a message that is too big for it to process.";
                else if(event.code == 1010) // Note that this status code is not used by the server, because it can fail the WebSocket handshake instead.
                    reason = "An endpoint (client) is terminating the connection because it has expected the server to negotiate one or more extension, but the server didn't return them in the response message of the WebSocket handshake. <br /> Specifically, the extensions that are needed are: " + event.reason;
                else if(event.code == 1011)
                    reason = "A server is terminating the connection because it encountered an unexpected condition that prevented it from fulfilling the request.";
                else if(event.code == 1015)
                    reason = "The connection was closed due to a failure to perform a TLS handshake (e.g., the server certificate can't be verified).";
                else
                    reason = "Unknown reason";
            }

            this._writeToLog("this._ws.onclose[code:"+event.code+", reason:"+reason+"]");

            if(isFunction(this.onWsClose)) this.onWsClose(event);
            this._ws=null;
          }.bindMe();

          this._ws.onerror = function (evt) {
            this._writeToLog("this._ws.onerror: "+evt.type);
            if(isFunction(this.onWsError)) this.onWsError(evt);
          }.bindMe();

        }.bindMe(),
        
        disconnect: function()
        {
            this._ws.close();
            //this._ws=null; // done on the onclose event handler
        }.bindMe(),
            
        //> Send a string message
        _sendMsg: function(msg) {
          if(this._ws===null) return;
          this._ws.send(msg);
        }, // pas de '.bindMe()' pour rester en private
             
        //> Send a ShortMessage Service
        sendSms1: function(msg)
        {
            if(!msg) return;
            var sms = {
                ShortMessageService: {
                    msgCode: null,
                    msgDesc: msg,
                    msgArgs:null,
                    msgTs : (new Date()).getTime()
                    }
            };
            var msgJSon = JSON.stringify(sms);
            this._sendMsg(msgJSon);
        }.bindMe(),
        
        extractReplyFromCall: function(stgyCallMsgBulk)
        {
            var msgBulk = {
                stgyReply:{
                    seq: stgyCallMsgBulk.stgyCall.seq,
                    token: stgyCallMsgBulk.stgyCall.token,
                    argsRef: stgyCallMsgBulk.stgyCall.argsRef
                }
            };
            return msgBulk;
        }.bindMe(),
                
        sendReply: function( msgBulk){
            if(!msgBulk) return;
            var msgJSon = JSON.stringify(msgBulk);
            this._sendMsg(msgJSon);
        }.bindMe(),
        
        
        Z: null
    };

    // declare the nameSpace
    window.DR2D_WSAPI = DR2D_WSAPI;

})();       

